/**************************************************************************//**
 * @file     low_power_detect.c
 * @version  V1.00
 * $Revision:
 * $Date: 14/08/15 6:09p $
 * @brief    Low power detect function source file
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include "M0518.h"
#include "action_script.h"
#include "function.h"

#define PLL_CLOCK       50000000
#define buffsize				4
/* battery threshold = 5*0.4/(3.3/2^12) almost euqal to 2480 */
#define battery_HT			2600				// Low battery high threshold for ADC value = battery threshold + 120
#define battery_LT			2360				// Low battery high threshold for ADC value = battery threshold - 120
/* power threshold = 3.3*0.4/(3.3/2^12) almost euqal to 1640 */
#define power_HT				1760				// Low power high threshold for ADC value = power threshold + 120
#define power_LT				1520				// Low power high threshold for ADC value = power threshold + 120
#define BT_LED					PC6					// BLUETOOTH_CONNECTION_LED < BLUETOOTH1
#define LowB_LED				PB2					// LOW_BATTERY_LED < BATLOW1
#define LowP_LED				PB3					// LOW_POWER_LED < CHIPLOW1
#define On							0
#define Off							1

extern uint8_t Command;

void low_battery_Init(void)
{

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init ADC & Timer Clock                                                                                  */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Enable peripheral clock */
		CLK_EnableModuleClock(TMR1_MODULE);
		CLK_EnableModuleClock(ADC_MODULE);

    /* Select peripheral clock source */
		CLK_SetModuleClock(TMR1_MODULE, CLK_CLKSEL1_TMR1_S_HCLK, 0);
		CLK_SetModuleClock(ADC_MODULE, CLK_CLKSEL1_ADC_S_HIRC, CLK_CLKDIV_ADC(7));

    /* Reset Funcction */
		SYS_ResetModule(TMR1_RST);
    SYS_ResetModule(ADC_RST);

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
		/* Disable the GPA4 digital input path to avoid the leakage current. */
    GPIO_DISABLE_DIGITAL_PATH(PA, 0x10);
    /* Set GPA.4/5 multi function pins for ADC 4~5 analog input pins */
    SYS->GPA_MFP &= ~(SYS_GPA_MFP_PA4_Msk | SYS_GPA_MFP_PA5_Msk);
    SYS->GPA_MFP |= (SYS_GPA_MFP_PA4_ADC4 | SYS_GPA_MFP_PA5_ADC5);

		/* Set the ADC operation mode as continuous scan, input mode as single-end and
					enable the analog input channel 4 and channel 5*/
		ADC_Open(ADC, ADC_ADCR_DIFFEN_SINGLE_END, ADC_ADCR_ADMD_CONTINUOUS, 0x30);

		/* Power on ADC module */
		ADC_POWER_ON(ADC);

		/* Clear the A/D interrupt flag for safe */
		ADC_CLR_INT_FLAG(ADC, ADC_ADF_INT);
	
		/* Open Timer1 in periodic mode, enable interrupt and 1 interrupt ticks per second */
	  TIMER_Open(TIMER1, TIMER_PERIODIC_MODE, 1);
    TIMER_EnableInt(TIMER1);

    /* Enable Timer1 NVIC */
    NVIC_EnableIRQ(TMR1_IRQn);	
		NVIC_SetPriority(TMR1_IRQn, 3);

    /* Start Timer1 counting */
    TIMER_Start(TIMER1);
		
		/* Low battery LED off*/
		LowB_LED = Off;
		/* Low power LED off*/
		LowP_LED = Off;

}

int low_battery_detect(void)
{
		int32_t  i32ConversionDataB;
		int32_t  i32ConversionDataP;
	
		/* Start A/D conversion */
		ADC_START_CONV(ADC);

		/* Wait conversion done */
		while(!ADC_GET_INT_FLAG(ADC, ADC_ADF_INT));

		/* Clear the A/D interrupt flag for safe */
		ADC_CLR_INT_FLAG(ADC, ADC_ADF_INT);

		/* Sparkle Low Battery LED if the ADC value is lower than high threshold */
		i32ConversionDataB = ADC_GET_CONVERSION_DATA(ADC, 4);
		/* Sparkle Low Power LED if the ADC value is lower than high threshold */		
		i32ConversionDataP = ADC_GET_CONVERSION_DATA(ADC, 5);
	
		if((i32ConversionDataB < battery_HT) | (i32ConversionDataP < power_HT))
		{
				/* Stop Action */
				Command = 0;
				Action_Init2();
			
				/* Sparkle LED */
				if(i32ConversionDataB < battery_HT)
						LowB_LED=On;
				if(i32ConversionDataP < power_HT)
						LowP_LED=On;				
				CLK_SysTickDelay(200000);
				LowB_LED=Off;
				LowP_LED=Off;
				
				/* Stop A/D conversion */
				ADC_STOP_CONV(ADC);
				return 1;
		}
		else
		{
				LowB_LED=Off;
				LowP_LED=Off;
			
				/* Stop A/D conversion */
				ADC_STOP_CONV(ADC);
				return 0;
		}
		
}

void TMR1_IRQHandler(void)
{
    if(TIMER_GetIntFlag(TIMER1) == 1)
    {
        /* Clear Timer1 time-out interrupt flag */
        TIMER_ClearIntFlag(TIMER1);

				/* Detect battery voltage and pulse buzzeer */
				if(low_battery_detect())
						Buzzer_Pulse(300000);				
    }
}

/*** (C) COPYRIGHT 2014 Nuvoton Technology Corp. ***/
